/**************************************************************************
**
** File: CHPASS.C     
**							 
** Desc: Sample demonstrating use of NWDSChangeObjectPassword
**		  	and NWDSGenerateObjectKeyPair.  The two ways to change a password
** 		in NDS.
**
**	Disclaimer:
**
**		Novell, Inc. makes no representations or warranties with respect to
**		any NetWare software, and specifically disclaims any express or
**		implied warranties of merchantability, title, or fitness for a
**		particular purpose.  
**
**		Distribution of any NetWare software is forbidden without the
**		express written consent of Novell, Inc.  Further, Novell reserves
**		the right to discontinue distribution of any NetWare software.
**
**		Novell is not responsible for lost profits or revenue, loss of use
**		of the software, loss of data, costs of re-creating lost data, the
**		cost of any substitute equipment or program, or claims by any party
**		other than you.  Novell strongly recommends a backup be made before
**		any software is installed.   Technical support for this software
**		may be provided at the discretion of Novell.
**
**	Description:
**
**		This example program demonstrates the two ways which a new
** 	password can be assigned to an object.  If the old password is supplied
** 	on the command line chpass.nlm logs in as that user and changes its
** 	password.  If the old password is not specified, a new key pair
** 	is generated with the new specified password.  In each case the result
** 	is the object is assigned the new password value.  This NLM does not
**		provide means to be unloaded at the console.  When providing an
**		administrator name, remember that this program uses default context.
**		This means normally that input can be typed or typeless but all names
**		will be referenced from the default name context set by the current
**		first bindery context.
**
**	QMK386 options used:
**
**		/id - Include NDS import file.
**		/z  - Generate for NLMLINKx.
**
** Syntax: load CHPASS <subject object> <new password> <old password>
**
** Programmers:
**
**    Ini   Who                  Firm
**    ---------------------------------------------------------------------
**    MDO   Mark D. Oberg			Novell Developer Support
**
**
** History:
**
** 	When		Who	What
** 	------------------------------------------------------------
** 	08-14-94 MDO	Begin
** 	08-18-94	MDO	Tested on 4.02 NetWare, 4.01e clib.
*/

/*************************************************************************
** Include header files
*/
	/*---------------------------------------------------------------------
	** NLMSDK includes
	*/
	#include <stdlib.h>
	#include <stdio.h>
	#include <string.h>
	#include <conio.h>
	#include <nwdsapi.h>
	#include <nwdsmisc.h>
	#include <nwdsdc.h>
	#include <nwconn.h>
	#include <nwenvrn.h>
	#include <nwcntask.h>

/*************************************************************************
** DisplayUsage
*/
void DisplayUsage(void)
	{
	printf("Syntax:  load chpass <subject object> <new password> <old password-optional>\n");
	printf("Example: load chpass .object.company newpassword oldpassword or\n");
	printf("Example: load chpass .object.company newpassword \n");
	exit(0);
	}

/*************************************************************************
** GetPassword prompts the user with "Password:" and accepts a string marking
** 	each entered position with an asterisk.
*/
void GetPassword(char *password, int size)
	{
	int	index=0;
	WORD	x,y;

	printf("Password: ");
	y=wherey();					/* find current row */

	do
		{
		x=wherex();				/* find current column */
		password[index]=getch();
		if(0x0D==password[index])
			password[index]='\0';

		else if(0x08==password[index] && 0<=index-1)	/* Backspace */
			{
			index-=2;
			gotoxy(x-1,y);			/* backup */
			putch(' ');				/* clear output */
			gotoxy(x-1,y);			/* backup */
			}
		else
			putch('*');

		index++;
		x++;			/* follow cursor position */
		}
	while (password[index-1] && index<size);
	if(password[index-1])
		password=NULL;
	putch('\n');
	}


/*************************************************************************
** main
*/
void main(int argc, char *argv[])
	{
	NWDSContextHandle	cx;
	NWDSCCODE	ccode;
	char	password[255+1];
	char	administrator[255+1];

	if(!(argc==4 || argc==3))
		{
		DisplayUsage();			/*display usage and exit nlm*/
		exit(0);
		}

	if(argc==3)
		{
		/*----------------------------------------------------------------------
		** requires an administrator to login because old password was not
		** provided
		*/
		printf("Login as user: ");
		gets(administrator);		/* get administrators name */

		GetPassword(password, 255); /* get password for administrator */
		clrscr();						 /* clear off user-password prompts */
		}

	cx=NWDSCreateContext();
	if(ERR_CONTEXT_CREATION==cx)			/* error */
		{
		printf("Error creating context.\n");
		exit(0);
		}

	ccode=NWDSLogin(
		/* NWDSContextHandle	context, */	cx,
		/* uint32	optionsFlag, 		*/	0,
		/* char NWFAR *objectName, 	*/	(argc == 3) ? administrator :argv[1],
		/* char NWFAR *password,		*/	(argc == 3) ? password : argv[3],
		/* uint32 validityPeriod		*/	60 );

	if(ccode)
		{
		if(argc==3)
			{
			printf("Unable to login as supplied administrator.\n\n");
			DisplayUsage();
			}
		else
			{
			printf("Old Password specified for object was incorrect.\n\n");
			DisplayUsage();
			}
		goto Exit1;
		}

	if(argc == 3)
		ccode=NWDSGenerateObjectKeyPair(
			/*	NWDSContextHandle	contextHandle, */ cx,
			/* char		NWFAR *objectName,		*/ argv[1],
			/* char		NWFAR *objectPassword,	*/ argv[2],	/* assign new password */
			/* uint32	optionsFlag					*/	0	);
	else
		ccode=NWDSChangeObjectPassword(
			/*	NWDSContextHandle		context,	*/	cx,
			/* NWDS_FLAGS optionsFlag,			*/ 0,
			/*	char		NWFAR *objectName,	*/	argv[1], 
			/*	char		NWFAR *oldPassword,	*/	argv[3],
			/*	char		NWFAR	*newPassword	*/	argv[2]	);

	if(ccode)
		{
		printf("Failed to change password.\n");
		printf("NWDSCCODE: %d",ccode);
		}
	else
		printf("Password Successfully changed.");

	NWDSLogout(cx);
Exit1:
	NWDSFreeContext(cx);
	exit(0);
	}

